﻿using Ektron.Cms.Find.Models;
using EPiServer.Find;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

public partial class FindSearch : System.Web.UI.UserControl
{
    #region Setup
    /// <summary>
    /// Querystring parameter for the keyword 
    /// </summary>
    public string qKey = "q";

    /// <summary>
    /// Querystring parameter for the order
    /// </summary>
    public string oKey = "o";

    //some code below adapted from http://joelabrahamsson.com/building-a-search-page-for-an-episerver-site-using-truffler-part-2/
    /// <summary>
    /// Querystring key for paging
    /// </summary>
    public string pKey = "p";

    /// <summary>
    /// Default page size
    /// </summary>
    public int PageSize = 10;
    private int _PageSize
    {
        get { return PageSize; }
        set { PageSize = value; }
    }
    /// <summary>
    /// Starting page number
    /// </summary>
    int pageNumber = 1;

    /// <summary>
    /// Item count for total number of search results
    /// </summary>
    public int ItemCount { get; set; }

    /// <summary>
    /// Retrieve the current page number
    /// </summary>
    public int ActivePage
    {
        get
        {
            if (!int.TryParse(
                Request.QueryString[pKey],
                out pageNumber))
            {
                pageNumber = 1;
            }
            return pageNumber;
        }
    }

    /// <summary>
    /// retrieve the total number of pages
    /// </summary>
    public int NumberOfPages
    {
        get { return (ItemCount + PageSize - 1) / PageSize; }
    }

    #endregion

    /// <summary>
    /// Helper to build out the page url for paging
    /// </summary>
    /// <param name="pageNumber">page number to be returned</param>
    /// <returns>returns full url of the page</returns>
    protected string GetPageUrl(int pageNumber)
    {
        if (!string.IsNullOrEmpty(Request.QueryString[pKey]))
        {
            var qs = HttpUtility.ParseQueryString(Request.QueryString.ToString());
            qs.Set(pKey, pageNumber.ToString());
            return Request.RawUrl.Substring(0, Request.RawUrl.IndexOf("?") + 1) + qs;
        }
        else
        {
            return Request.RawUrl + "&" + pKey + "=" + pageNumber;
        }
    }

    /// <summary>
    /// Page load event to determine if we should fire off a search
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        //check for postback since changing sort order would fire off another search
        if (!IsPostBack)
        {
            //make sure there is a query for search
            if (!string.IsNullOrEmpty(Request.QueryString[qKey]))
            {
                //execute the search
                Search();
            }
        }
    }

    /// <summary>
    /// Search the site
    /// </summary>
    private void Search()
    {
        try
        {
            //set the search textbox with the keyword provided in the querystring
            searchKeyword.Text = Request.QueryString[qKey];
            //check to make sure our query length is greater than 2 so we aren't searching on small words
            if (Request.QueryString[qKey].Length > 2)
            {
                //create new instance of search client
                var client = Client.CreateFromConfig();
                //build out our query object
                var query = client.Search<Ektron.Cms.Find.Models.Content>()
                    .For(Request.QueryString[qKey])
                    //.UsingSynonyms()
                    .Take(PageSize)
                    .Skip((ActivePage - 1) * PageSize);
                //read order key from querystring
                if (!string.IsNullOrEmpty(Request.QueryString[oKey]))
                {
                    switch (Request.QueryString[oKey])
                    {
                        case "2":
                            //order by Title
                            query = query.OrderBy(a => a.Title);
                            orderBy.SelectedValue = "2";
                            break;
                        case "3":
                            //order by Last Modification Date
                            query = query.OrderBy(a => a.ModificationDate);
                            orderBy.SelectedValue = "3";
                            break;
                        default:
                            //order by Relevance
                            orderBy.SelectedValue = "1";
                            break;
                    }
                }
                else
                {
                    //default sort ordering of Find is Relevance
                    orderBy.SelectedValue = "1";
                }
                //retrieve search results
                var results = query.GetResult();
                //databind our search results to the repeater
                ResultsRepeater.DataSource = results;
                ResultsRepeater.DataBind();
                //set item count for use with pagination
                ItemCount = results.TotalMatching;
                //if there are no matches, hide controls otherwise show order by dropdown
                if (results.TotalMatching == 0)
                {
                    nosearchresults.Visible = true;
                    ResultsRepeater.Visible = false;
                    pagination.Visible = false;
                }
                else
                {
                    orderBy.Visible = true;
                }
            }
            else
            {
                //show no results if querystring length is less than 2
                nosearchresults.Visible = true;
                ResultsRepeater.Visible = false;
                pagination.Visible = false;
                orderBy.Visible = false;
            }
        }
        catch (Exception ex)
        {
            //show no results if exception occurs
            nosearchresults.Visible = true;
            ResultsRepeater.Visible = false;
            pagination.Visible = false;
            orderBy.Visible = false;
        }
    }

    /// <summary>
    /// Search after search button is clicked
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void searchBtn_Click(object sender, EventArgs e)
    {
        //check for a querystring parameter of keyword key to see if query already exists
        if (Request.RawUrl.Contains("?" + qKey))
        {
            //build querystring
            Response.Redirect(Request.RawUrl.Substring(0, Request.RawUrl.IndexOf("?")) + "?" + qKey + "=" + searchKeyword.Text);
        }
        else
        {
            //build querystring
            Response.Redirect(Request.RawUrl + "?" + qKey + "=" + searchKeyword.Text);
        }
    }

    /// <summary>
    /// Change the sort ordering of the results
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void orderBy_SelectedIndexChanged(object sender, EventArgs e)
    {
        //check for a querystring parameter of keyword key to see if query already exists
        if (Request.RawUrl.Contains("?" + qKey))
        {
            //build querystring
            Response.Redirect(Request.RawUrl.Substring(0, Request.RawUrl.IndexOf("?")) + "?" + qKey + "=" + searchKeyword.Text + "&" + oKey + "=" + orderBy.SelectedValue);
        }
        else
        {
            //build querystring
            Response.Redirect(Request.RawUrl + "?" + qKey + "=" + searchKeyword.Text + "&" + oKey + "=" + orderBy.SelectedValue);
        }
    }
}
